# sb::App::Main
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::App::Main;
use strict;
use sb::Data ();
use sb::Text ();
use sb::Content ();
use sb::App::Receipt ();
use sb::App ();
use base qw( sb::App );
use vars qw( $VERSION );
$VERSION = '0.14';
# 0.14 [2007/10/01] modified _check_mode to consider uncategoried entry (cid == -1)
# 0.13 [2007/09/18] fixed some error messages
# 0.12 [2007/09/17] fixed a bug in _check_mode template cannot be specified
# 0.11 [2007/08/19] debug flag can be enabled by Config
# 0.10 [2007/08/12] enabled receipt, added debug flag of content
# 0.09 [2007/08/04] changed initialize
# 0.08 [2007/07/20] refactored for Serene Bach 3.*
# 0.07 [2006/12/15] changed _check_mode to fix a bug
# 0.06 [2006/02/15] changed _check_mode to check chaging style correctly
# 0.05 [2005/10/19] changed _check_mode to check permission correctly
# 0.04 [2005/10/18] changed _check_mode to pass TemplateManager object instead of template contents.
# 0.03 [2005/08/23] changed _check_mode to change template correctly
# 0.02 [2005/07/11] changed config variable "srv_cgi" to "conf_srv_cgi"
# 0.01 [2005/07/08] changed run to set cookie correctly
# 0.00 [2005/02/01] generated
# --------------------------------------
# // defines / static variables
sub CONTENT_TYPE_HTML (){ 'text/html' };
sub CONTENT_TYPE_CSS  (){ 'text/css' };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	return $self->SUPER::initialize(
		'mode' => 'page',
		'base' => undef,
		'css'  => undef,
		'cond' => '',
		'id'   => undef,
		@_
	);
}
sub run
{
	my $class = shift;
	my $self = $class->new(@_);
	$self->_check_mode();
	if ($self->{'mode'} eq 'tb' or $self->{'mode'} eq 'com')
	{
		sb::App::Receipt->new(
			'time' => $self->time(),
			'mode' => $self->{'mode'},
			'id'   => $self->{'id'},
		)->issue();
	}
	else
	{
		my $cookie = $self->cgi->cookie('name'=>$self->conf->value('basic_cooktag'));
		$cookie->{'checkid'} = $self->conf->value('conf_spamid');
		$self->cgi->set_cookie(
			'time'   => $self->time->data(),
			'name'   => $self->conf->value('basic_cooktag'),
			'expire' => 365,
			'data'   => $cookie,
		);
		my $content = sb::Content->new();
		$content->debug( $self->conf->value('debug_content') );
		my $out = $content->output($self->{'base'},
			'mode' => $self->{'mode'},
			'css'  => $self->{'css'},
			'id'   => $self->{'id'},
			'page' => int( $self->cgi->value('page') ),
			'cond' => $self->{'cond'},
			'time' => $self->time->data(),
		);
		$self->cgi->head(
			'type'   => ($self->{'mode'} eq 'css') ? CONTENT_TYPE_CSS : CONTENT_TYPE_HTML,
			'length' => ($content->debug) ? undef : length($out),
		);
		if ($content->error)
		{
			warn($content->error);
			$out .= '<pre>' . $content->error . '</pre>';
		}
		$self->output($out);
	}
}
# --------------------------------------
# // private functions
sub _check_mode
{
	my $self = shift;
	# -- receiving trackback or comment
	my $received = $self->cgi->value('tb');
	if ($self->cgi->path_info() ne '')
	{
		$received = $self->cgi->path_info();
		$received =~ s/.*[:\/\\](.*)/$1/;
	}
	$received = $self->cgi->value('entry_id') if ($self->cgi->value('entry_id') ne '');
	if ($received =~ /^\d+$/)
	{
		$self->{'id'} = $received;
		$self->{'mode'} = ($self->cgi->value('entry_id') ne '') ? 'com' : 'tb';
		return;
	}
	# -- displaying stylesheet
	if ($self->cgi->value('css') =~ /^\d+$/)
	{
		$self->{'mode'} = 'css';
		$self->{'base'} = $self->_get_template($self->cgi->value('css'));
		return;
	}
	# -- dynamic publishing
	my $tid = undef;
	my $cid = undef;
	my $flag = undef; # flag for changing template
	$self->{'mode'} = 'ent'  if ($self->cgi->value('eid') =~ /^\d+$/);   # entry mode
	$self->{'mode'} = 'user' if ($self->cgi->value('pid') =~ /^\d+$/);   # profile mode
	$self->{'mode'} = 'cat'  if ($self->cgi->value('cid') =~ /^\d+$/);   # category mode
	$self->{'mode'} = 'arc'  if ($self->cgi->value('month') =~ /^\d+$/); # monthly archive mode
	$self->{'mode'} = 'arc'  if ($self->cgi->value('day') =~ /^\d+$/);   # daily archive mode
	$self->{'mode'} = 'srch' if ($self->cgi->value('search') ne '');     # search
	# -- checking entry and category
	$cid = int($self->cgi->value('cid')) if ($self->{'mode'} eq 'cat');
	if ($self->{'mode'} eq 'ent')
	{
		my $entry = sb::Data->load('Entry','id'=>int($self->cgi->value('eid')));
		die(sb->string('No article.') . "\n") if (!$entry);
		$cid = $entry->cat;
		if ($entry->stat == 0)
		{
			my $cookie = $self->cgi->cookie('name'=>$self->conf->value('basic_logtag'));
			if ($cookie->{'check'} ne $self->conf->value('basic_cookiekey'))
			{
				die(sb->string('No article.') . "\n");
			}
		}
	}
	# -- checking template
	$tid = int($self->cgi->value('tid')) if ($self->cgi->value('tid') ne '');
	if ( $tid eq '' )
	{ # no specific template
		if ( $cid ne '' && $cid != -1 )
		{ # getting the category template
			my $category = sb::Data->load('Category','id'=>$cid);
			$tid = $category->temp if ($category->temp ne '' and $category->temp > -1);
		}
		if ( $tid eq '' )
		{ # we didn't still have any template
			if ($self->{'mode'} =~ /^[srch|arc|cat]$/ and $self->conf->value('conf_archive_temp') > -1)
			{
				$tid = $self->conf->value('conf_archive_temp');
			}
			elsif ($self->{'mode'} eq 'user' and $self->conf->value('conf_profile_temp') > -1)
			{
				$tid = $self->conf->value('conf_profile_temp');
			}
		}
		# template is specified, but it's not forced.
		$flag = 1 if ($tid ne '');
	}
	$self->{'base'} = $self->_get_template($tid);
	# -- getting css path
	$self->{'css'} = $self->conf->value('conf_srv_base') . $self->conf->value('file_css');
	if ( $tid ne '' and (!$flag or $self->conf->value('conf_css_change')))
	{
		$self->{'css'} = $self->conf->value('conf_srv_cgi') . $self->conf->value('basic_sb') . '?css=' . $tid;
	}
	# -- getting id
	$self->{'id'} = 
		  ($self->{'mode'} eq 'ent')  ? int($self->cgi->value('eid'))
		: ($self->{'mode'} eq 'user') ? int($self->cgi->value('pid'))
		: ($self->{'mode'} eq 'cat')  ? int($self->cgi->value('cid'))
		:                              undef;
	# -- getting condition of archive
	$self->{'cond'} = 
		  ($self->cgi->value('month') =~ /^\d+$/) ? $self->cgi->value('month')
		: ($self->cgi->value('day') =~ /^\d+$/)   ? $self->cgi->value('day')
		: ($self->{'mode'} eq 'srch')             ? sb::Text->new($self->cgi->value('search'))->entitized()
		:                                           undef;
	return;
}
sub _get_template
{
	my $self = shift;
	my $preferred = shift;
	my $mode = $self->{'mode'};
	my $template = undef;
	$mode = 'entry' if ( $mode =~ /^ent/);
	$mode = 'main' if ($mode ne 'entry' and $mode ne 'css');
	if ( defined($preferred) )
	{
		$template = sb::Data->load('Template',id=>$preferred);
	}
	if ( !$template )
	{
		$template = sb::Data->load('Template','cond'=>{'use'=>1},'num'=>1);
	}
	die(sb->string('Failed to get a template.') . "\n") if ( !$template );
	$mode = 'main' if ($mode eq 'entry' and $template->entry eq '');
	return sb::Template->new($template->$mode());
}
1;
__END__
