# sb::App::Rsd
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::App::Rsd;
use strict;
use sb::Data ();
use sb::Content ();
use sb::App ();
use base qw( sb::App );
use vars qw( $VERSION );
$VERSION = '0.03';
# 0.03 [2007/08/16] changed run to use new instead of SUPER->new
# 0.02 [2007/08/14] modified for Serene Bach 3.*
# 0.01 [2005/10/18] changed run to pass TemplateManager object to sb::Content
# 0.00 [2005/08/11] generated
# --------------------------------------
# // defines / static variables
sub TEMPLATE_FILE  (){ 'default_rsd.xml' };
sub CONTENT_TYPE   (){ 'text/xml' };
sub OUTPUT_CHARSET (){ 'utf-8' };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	return $self->SUPER::initialize(@_);
}
sub run
{
	my $class = shift;
	my $self = $class->new(@_);
	my $content = sb::Content->new(
		'main' => {
			'_rsd' => \&_add_entry_urls,
		},
	);
	$content->debug(1); # for debug
	my $output = $self->load_template('file'=>TEMPLATE_FILE);
	if ($output ne '')
	{
		$output = $content->output(sb::Template->new($output),
			'mode'      => 'page',
			'time'      => $self->time->data(),
			'cat'       => {}, # dummy
			'entry'     => {}, # dummy
			'entry_num' => 0,
		);
	}
	warn($content->error()) if ($content->error);
	$self->cgi->head(
		'type'    => CONTENT_TYPE,
		'charset' => OUTPUT_CHARSET,
		'length'  => ($content->debug) ? undef : length($output),
	);
	$self->output($output);
}
# --------------------------------------
# // private functions - content
sub _add_entry_urls
{
	my $cms = shift;
	my %var = @_;
	$cms->tag('rsd_xml_entry_url'=>$var{'conf'}->value('conf_srv_cgi') . $var{'conf'}->value('basic_xmlrpc'));
}
1;
=head1 NAME

sb::App::Rsd - Serene Bach application for Really Simple Discoverability

=head1 SYNOPSIS

	require sb::App::Rsd;
	sb::App::Rsd->run();

=head1 DESCRIPTION

sb::App::Rsd output Really Simple Discoverability as XML.
ref. http://archipelago.phrasewise.com/rsd

Normally this app is indicated as follow in <head>.
	<link rel="EditURI" type="application/rsd+xml" title="RSD" href="{site_rsd}" />

=head1 AUTHOR

Takuya Otani http://serenebach.net/

=head1 LICENSE

Copyright (C) 2004- Takuya Otani(SimpleBoxes) and SerendipityNZ

=cut
__END__
