#!/usr/bin/perl
# 
# test script for Serene Bach libraries
# == written by T.Otani <ootani@segausers.gr.jp> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

# ======================================
use strict;
# use warnings;
use lib qw(./ ../);
use sb::Test;
sb::Test::InitParser->run();

package sb::Test::InitParser;
use vars qw( $VERSION );
$VERSION = '0.02';
# 0.02 [2007/07/17] _reset_values is no longer needed
# 0.01 [2007/07/16] changed return to make them consistent
# 0.00 [2007/07/13] created
# --------------------------------------
# // public functions
sub run
{
	my $testcase = sb::Test->new();
	$testcase->add('initparser',\&_test,\&_check);
	$testcase->run();
}
# --------------------------------------
# // private functions
sub _test
{
	require sb::InitParser;
	return {} if (! sb::InitParser->new('file'=>_filename())->save(_data()) );
	my $data = {};
	return {} if (! sb::InitParser->new('file'=>_filename())->load($data));
	return $data;
}
sub _check
{
	my $result = shift;
	unlink(_filename()) if (-e _filename()); # delete temporary file to tidy up
	return if (!$result);
	return if (ref($result) ne 'HASH');
	my $orig = _data();
	foreach my $key ( keys %{$orig} )
	{
		return if (! defined($result->{$key}));
		if (ref($orig->{$key}) eq 'HASH')
		{
			return if (ref($result->{$key}) ne 'HASH');
			foreach my $hkey ( keys %{$orig->{$key}} )
			{
				return if (! defined($result->{$key}->{$hkey}));
				return if ($orig->{$key}->{$hkey} ne $result->{$key}->{$hkey});
			}
		}
		elsif (ref($orig->{$key}) eq 'ARRAY')
		{
			return if (ref($result->{$key}) ne 'ARRAY');
			for my $i ( 0 .. $#{$orig->{$key}} )
			{
				return if (! defined($result->{$key}->[$i]));
				return if ($orig->{$key}->[$i] ne $result->{$key}->[$i]);
			}
		}
		elsif ($orig->{$key} ne $result->{$key})
		{
			return;
		}
	}
	return 1;
}
sub _data
{
	return {
		'hash'          => {'key1'=>'value1','key2'=>'value2'},
		'hashwithnoval' => {'key1'=>undef,'key2'=>'check'},
		'emptyhash'     => {},
		'array'         => ['value1','value2'],
		'emptyarray'    => [],
		'string'        => 'value',
		'integer'       => 100,
		'zero'          => 0,
		'undef'         => undef,
		'special_chars' => "test\ttest\n",
	};
}
sub _filename
{
	return 'temp_conf.log';
}
1;
__END__
